﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  public enum ProducedBy { Crossover, Mutation };

  [Item("PermutationInformation", "Stores meta-information about a solution.")]
  [StorableClass]
  public class PermutationInformation : Item {
    [Storable]
    public ProducedBy ProducedBy { get; set; }
    [Storable]
    public List<PermutationWrapper> ParentList { get; set; }
    [Storable]
    public int Generation { get; set; }
    [Storable]
    PermutationWrapperEqualityComparer pweq;

    public PermutationInformation() {
      pweq = new PermutationWrapperEqualityComparer();
    }

    [StorableConstructor]
    protected PermutationInformation(bool deserializing) : base(deserializing) { }
    protected PermutationInformation(PermutationInformation original, Cloner cloner)
      : base(original, cloner) {
      this.ProducedBy = original.ProducedBy;
      this.ParentList = new List<PermutationWrapper>(original.ParentList);
      this.Generation = original.Generation;
      this.pweq = new PermutationWrapperEqualityComparer();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PermutationInformation(this, cloner);
    }

    public override int GetHashCode() {
      //use rotating hash, should be enough
      //see: http://eternallyconfuzzled.com/tuts/algorithms/jsw_tut_hashing.aspx
      int h = 0;

      h = (h << 4) ^ (h >> 28) ^ ProducedBy.GetHashCode();
      h = (h << 4) ^ (h >> 28) ^ Generation.GetHashCode();

      if (ParentList != null) {
        foreach (var parent in ParentList) {
          h = (h << 4) ^ (h >> 28) ^ pweq.GetHashCode(parent);
        }
      }

      return h;
    }

    public override bool Equals(object obj) {
      PermutationInformation pi = obj as PermutationInformation;

      if (ProducedBy == pi.ProducedBy && Generation == pi.Generation) {
        if (pi.ParentList == null && ParentList == null) return true;
        if (pi.ParentList != null && ParentList == null ||
          pi.ParentList == null && ParentList != null) return false;
        if (pi.ParentList.Count != ParentList.Count) return false;

        foreach (var p in ParentList) {
          if (!pi.ParentList.Contains(p)) return false;
        }
        return true;
      } else {
        return false;
      }
    }
  }
}
